#!/bin/bash

#set -x

#
# this script installs "our" packages
# it just needs the name as a parameter
# 

mypath="/var/lib/ctsrvcfg"
generics="$mypath/generic"
myvars="/tmp/myvars.$$"
package=$1
installpath=`find $mypath -name "$package" -a -type d`

pushd $installpath > /dev/null

# get a list of webminable packages
webminable=`apt-cache search webmin | cut -d "-" -f2`

# be sure to stop running service (ipcop special)
# (ipcop image files will be loop mounted here!)
if [ -e "./pre_set1.sh" ]; then
    ./pre_set1.sh $MNTPT
fi

# keep daemons from starting
SSD="start-stop-daemon"
mv /sbin/$SSD /sbin/$SSD.REAL
echo "\
#!/bin/bash
echo
echo \"Warning: Fake $SSD called, doing nothing\"" > /sbin/$SSD 
chmod 755 /sbin/$SSD

# we dont't like to see any debconf questions here
export DEBIAN_PRIORITY=critical

# preseed any questions
if [ -e "./preseed" ]; then
    debconf-set-selections ./preseed
fi

# fake running daemon (dhcp3-server "needs" it
# it just creates the needed pid file in /var/run)
if [ -e "./fakerun_on.sh" ]; then
    ./fakerun_on.sh
fi

# may install more than just selected package
addpkgs=""

# check if it manually pulls other packages
# mostly those are recommends ...
# check if manually pulls its webmin-module
# becaue of naming differences
addpkgs=`cat ./addpkgs ./webmin 2> /dev/null`

# check if there is a auto webmin module
if [ ! -e "./webmin" ]; then
    if echo "$webminable" | grep -q "$package"; then
	addpkgs="$addpkgs webmin-$package"
    fi
fi

# install it
# TODO: let apt take maintainer conf files
# (that's what we need for cfengine2 scripts
apt-get install $package $addpkgs -y
APTRES="$?"

# unfake it
if [ -e "./fakerun_off.sh" ]; then
    ./fakerun_off.sh
fi

# restore start-stop-daemon
mv -f /sbin/$SSD.REAL /sbin/$SSD

# after restoring all, time to check
# apt's success ...
if [ "$APTRES" != "0" ]; then
	echo "apt-get install failed on: $package $addpkgs"
	exit 1
fi

# special for ipcop - need to mount root
# image to configure; it's just mountable now
# furthermore it needs to stop networking
# that fits in perfectly here
if [ -e "./pre_set2.sh" ]; then
    export MNTPT="/tmp/mountme.$$"
    mkdir $MNTPT
    ./pre_set2.sh $MNTPT
fi

# get common vars (ip parameters, timestamp and the lot)
. $generics/getcommon.sh

# read current package's debconf variables into shell space
# they may overwrite common vars !
if [ -e "./vars" ]; then 
    $generics/read_and_export_debconf_vars.sh ./vars $myvars
    # get them exported
    . $myvars
fi

# let cfengine2 generate the default config file(s)
if [ -e "./default.cfe2" ]; then
    cfagent -v -f ./default.cfe2 >> /tmp/cfoutput.log 2>&1
fi

# let cfengine2 adapt config files as needed
if [ -e "./set.cfe2" ]; then
    cfagent -v -f ./set.cfe2 >> /tmp/cfoutput.log 2>&1
fi

# check if this package has some sub-options
# that need some defaults pushed in by cfengine2
subopts=`find . -type d -maxdepth 1 | grep -v "^.$" | sed 's|./||g'`
for s in $subopts; do
    subdirs=`find $s -type d | wc -l`
    # got one
    if [ ! -e "$s/.pkg" ] && [ "$subdirs" = "1" ]; then
        adddefaults="$s $adddefaults"
    fi
done

# do the same for any sub-option
for a in $adddefaults; do

    pushd $a > /dev/null
    # read current package's debconf variables into shell space
    # they may overwrite common vars !
    if [ -e "./vars" ]; then 
	$generics/read_and_export_debconf_vars.sh ./vars $myvars
        # get them exported
	. $myvars
    fi

    # let cfengine2 generate the default config file(s)
    if [ -e "./default.cfe2" ]; then
        cfagent -v -f ./default.cfe2 >> /tmp/cfoutput.log 2>&1
    fi

    # let cfengine2 adapt config files as needed
    if [ -e "./set.cfe2" ]; then
        cfagent -v -f ./set.cfe2 >> /tmp/cfoutput.log 2>&1
    fi
    popd > /dev/null
done

# unloopmount (ipcop special, restart is 
# done later with initcall) 
if [ -e "./pre_set2.sh" ]; then
    umount $MNTPT
fi

# now the daemon may be started
# packages may do other things here
# (ipcop does so with starting network)
if [ -e "./initcall.sh" ]; then
    ./initcall.sh start
else
    if [ -e "/etc/init.d/$package" ]; then
	/etc/init.d/$package start
    fi
fi

# some packages need some more configuration
# data feed in if they are running after install
# (cyrus21-imapd for example to create mailboxes)
if [ -e "./late.cfe2" ]; then
    cfagent -v -f ./late.cfe2 >> /tmp/cfoutput.log 2>&1
fi

# clean up
rm -f $myvars
[ -d "$MNTPT" ] && rmdir $MNTPT
